{-|
Module      : EPO.API.Client
Description : EPO Patents database crawler (API client)
Copyright   : (c) CNRS, 2023-present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}

module EPO.API.Client where

import Data.Text qualified as T
import EPO.API.Client.Types
import Network.HTTP.Client (defaultManagerSettings, newManager, requestHeaders)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Network.URI qualified as URI
import Protolude
import Servant.API
import Servant.Client


defaultClientEnv :: URI.URI -> IO ClientEnv
defaultClientEnv uri  = do
  let (Just URI.URIAuth { uriRegName, uriPort }) = URI.uriAuthority uri
  let (proto, port, managerSettings) =
        if URI.uriScheme uri == "https:" then
          (Https, 443, tlsManagerSettings)
        else
          (Http, 80, defaultManagerSettings)
  let uriPort' =
        if uriPort == ""
        then Right port
        else (readEither (T.unpack $ T.tail $ T.pack uriPort) :: Either [Char] Int)
  manager <- newManager managerSettings
  let env' = mkClientEnv manager $ BaseUrl proto uriRegName (fromRight 80 uriPort') ""
  let makeClientRequest bu req =
        let r = defaultMakeClientRequest bu req in
        r { requestHeaders = (requestHeaders r) <> [("User-Agent", "servant.client")]}
  let env = env' { makeClientRequest }
  -- pure $ addLoggingToClientEnv env
  pure env



type EPOAPI =
  "api"
  :> "search"
    :> Header "Authorization" AuthKey
    :> QueryParam "start" Start
    :> QueryParam "end" End
    :> QueryParam "s" Search
    :> Get '[JSON] HyperdataDocuments


epoApi :: Proxy EPOAPI
epoApi = Proxy

search :: Maybe AuthKey
       -> Maybe Int
       -> Maybe Int
       -> Maybe Text
       -> ClientM (Paginated HyperdataDocument)

search = client epoApi
